@@ -1,9 +1,21 @@
 #!/usr/bin/env python
-import threading, Queue, time, re, subprocess, tempfile
+
+from __future__ import print_function
+
+import threading
+try:
+    from queue import Empty, Queue
+except ImportError:
+    from Queue import Empty, Queue
+import time
+import re
+import subprocess
+import os
 import aptsources
-from timeit import Timer
-import urllib2
-import socket
+try:
+    from urllib.request import urlopen
+except ImportError:
+    from urllib2 import urlopen
 import random
 
 class MirrorTest(threading.Thread):
@@ -31,13 +43,14 @@ class MirrorTest(threading.Thread):
                     self.parent.report_action("Pinging %s..." % host)
                     commando = subprocess.Popen("ping -q -c 2 -W 1 -i 0.5 %s" % host,
                                                 shell=True, stdout=subprocess.PIPE,
-                                                stderr=subprocess.STDOUT).stdout
+                                                stderr=subprocess.STDOUT,
+                                                universal_newlines=True).stdout
                     while True:
                         line = commando.readline()
                         if not line:
                             break
                         result = re.findall(self.match_result, line)
-                except Queue.Empty, e:
+                except Empty:
                     return
                 except:
                     self.parent.report_action("Skipping %s" % host)
@@ -52,8 +65,12 @@ class MirrorTest(threading.Thread):
                     self.results.append([float(result[0]), host, mirror])
                 MirrorTest.completed_lock.release()
 
-    def __init__(self, mirrors, test_file, running=None):
+    def __init__(self, mirrors, test_file, event, running=None):
         threading.Thread.__init__(self)
+        self.action = ''
+        self.progress = (0, 0, 0.0) # cur, max, %
+        self.event = event
+        self.best = None
         self.test_file = test_file
         self.threads = []
         MirrorTest.completed = 0
@@ -66,14 +83,18 @@ class MirrorTest(threading.Thread):
             self.running = running
 
     def report_action(self, text):
-        """Subclasses should override this method to receive
-           action message updates"""
-        print text
+        self.action = text
+        if self.event:
+            self.event.set()
 
     def report_progress(self, current, max, borders=(0,100), mod=(0,0)):
         """Subclasses should override this method to receive
            progress status updates"""
-        print "Completed %s of %s" % (current + mod[0], max + mod[1])
+        self.progress = (current + mod[0], 
+                         max + mod[1],
+                         borders[0] + (borders[1] - borders[0]) / max * current)
+        if self.event:
+            self.event.set()
 
     def run_full_test(self):
         # Determinate the 5 top ping servers
@@ -83,17 +104,17 @@ class MirrorTest(threading.Thread):
         if size > 2:
             results_ping.append([0, 0, self.mirrors[random.randint(1, size-1)]])
             results_ping.append([0, 0, self.mirrors[random.randint(1, size-1)]])
-        results = self.run_download_test(map(lambda r: r[2], results_ping),
+        results = self.run_download_test([r[2] for r in results_ping],
                                          borders=(0.5, 1),
                                          mod=(MirrorTest.todo,
                                               MirrorTest.todo))
         for (t, h) in results:
-            print "mirror: %s - time: %s" % (h.hostname, t)
+            print("mirror: %s - time: %s" % (h.hostname, t))
         if not results:
             return None
         else:
             winner = results[0][1].hostname
-            print "and the winner is: %s" % winner
+            print("and the winner is: %s" % winner)
             return winner
 
     def run_ping_test(self, mirrors=None, max=None, borders=(0,1), mod=(0,0)):
@@ -103,7 +124,7 @@ class MirrorTest(threading.Thread):
            the download test is only a part of a whole series of tests."""
         if mirrors == None:
             mirrors = self.mirrors
-        jobs = Queue.Queue()
+        jobs = Queue()
         for m in mirrors:
             jobs.put(m)
         results = []
@@ -131,7 +152,7 @@ class MirrorTest(threading.Thread):
             self.report_action("Downloading %s..." % url)
             start = time.time()
             try:
-                data = urllib2.urlopen(url, timeout=2).read(102400)
+                urlopen(url, timeout=2).read(102400)
                 return time.time() - start
             except:
                 return 0
@@ -149,6 +170,11 @@ class MirrorTest(threading.Thread):
         results.sort()
         return results[0:max]
 
+    def run(self):
+        """Complete test exercise, set self.best when done"""
+        self.best = self.run_full_test()
+        self.running.clear()
+
 if __name__ == "__main__":
     distro = aptsources.distro.get_distro()
     distro.get_sources(aptsources.SourcesList())
@@ -158,6 +184,6 @@ if __name__ == "__main__":
                 (distro.source_template.name,
                  distro.source_template.components[0].name,
                  arch)
-    app = MirrorTest(distro.source_template.mirror_set.values(),
+    app = MirrorTest(list(distro.source_template.mirror_set.values()),
                      test_file)
     app.run_full_test()
