@@ -22,36 +22,43 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
+from __future__ import absolute_import, print_function
+
 import apt
 import apt_pkg
 from hashlib import md5
 import re
 import os
 import glob
-import inspect
-import sys
 import shutil
-import subprocess
 import threading
 import atexit
 import tempfile
-import string
+try:
+  from string import maketrans
+except ImportError:
+  maketrans = str.maketrans
 import stat
 
 from tempfile import NamedTemporaryFile
 from xml.sax.saxutils import escape
-from ConfigParser import ConfigParser
+try:
+  from configparser import ConfigParser
+except ImportError:
+  from ConfigParser import ConfigParser
 from gettext import gettext as _
-from urlparse import urlparse
+try:
+  from urllib.parse import urlparse
+except ImportError:
+  from urlparse import urlparse
 
 import aptsources
 import aptsources.distro
 import softwareproperties
 
-from AptAuth import AptAuth
+from .AptAuth import AptAuth
 from aptsources.sourceslist import SourcesList, SourceEntry
-from ppa import AddPPASigningKeyThread, expand_ppa_line
-from cloudarchive import shortcut_handler as ca_shortcut_handler
+from .ppa import AddPPASigningKeyThread, expand_ppa_line
 
 class SoftwareProperties(object):
 
@@ -65,7 +72,7 @@ class SoftwareProperties(object):
     RELEASE_UPGRADES_NORMAL,
     RELEASE_UPGRADES_LTS,
     RELEASE_UPGRADES_NEVER
-  ) = range(3)
+  ) = list(range(3))
   release_upgrades_policy_map = {
     RELEASE_UPGRADES_NORMAL : 'normal',
     RELEASE_UPGRADES_LTS    : 'lts',
@@ -196,7 +203,7 @@ class SoftwareProperties(object):
     parser.read(self.RELEASE_UPGRADES_CONF)
     if parser.has_option("DEFAULT","Prompt"):
       type = parser.get("DEFAULT","Prompt").lower()
-      for k, v in self.release_upgrades_policy_map.iteritems():
+      for k, v in self.release_upgrades_policy_map.items():
         if v == type:
           return k
     return self.RELEASE_UPGRADES_NORMAL
@@ -211,13 +218,14 @@ class SoftwareProperties(object):
     # we are note using ConfigParser.write() as it removes comments
     if not os.path.exists(self.RELEASE_UPGRADES_CONF):
       f = open(self.RELEASE_UPGRADES_CONF,"w")
-      f.write("[DEFAULT]\nprompt=%s\n"% self.release_upgrades_policy_map[i])
+      f.write("[DEFAULT]\nPrompt=%s\n"% self.release_upgrades_policy_map[i])
       return True
     f = open(self.RELEASE_UPGRADES_CONF,"r")
-    out = NamedTemporaryFile()
-    for line in map(string.strip, f.readlines()):
+    out = NamedTemporaryFile(mode="w+")
+    for line in f:
+      line = line.strip()
       if line.lower().startswith("prompt"):
-        out.write("prompt=%s\n" % self.release_upgrades_policy_map[i])
+        out.write("Prompt=%s\n" % self.release_upgrades_policy_map[i])
       else:
         out.write(line+"\n")
     out.flush()
@@ -252,14 +260,13 @@ class SoftwareProperties(object):
     if os.path.exists(self.popconfile):
         # read the current config and replace the corresponding settings
         # FIXME: should we check the other values, too?
-        lines = map(lambda line: re.sub(r'^(PARTICIPATE=)(".+?")',
-                                        '\\1"%s"' % value,
-                                        line),
-                    open(self.popconfile, "r").readlines())
+        with open(self.popconfile, "r") as popconfile:
+            lines = [re.sub(r'^(PARTICIPATE=)(".+?")', '\\1"%s"' % value, line)
+                     for line in popconfile]
     else:
         # create a new popcon config file
         m = md5()
-        m.update(open("/dev/urandom", "r").read(1024))
+        m.update(open("/dev/urandom", "rb").read(1024))
         id = m.hexdigest()
         lines = []
         lines.append("MY_HOSTID=\"%s\"\n" % id)
@@ -287,7 +294,7 @@ class SoftwareProperties(object):
         sources.extend(self.distro.main_sources)
         sources.extend(self.distro.child_sources)
         for source in sources:
-            if templates.has_key(source.template):
+            if source.template in templates:
                 for comp in source.comps:
                     templates[source.template].add(comp)
             else:
@@ -297,8 +304,8 @@ class SoftwareProperties(object):
         if len(self.distro.cdrom_sources) > 0:
             templates[self.distro.source_template] = self.distro.cdrom_comps
         for source in self.distro.source_code_sources:
-            if not templates.has_key(source.template) or \
-               (templates.has_key(source.template) and not \
+            if source.template not in templates or \
+               (source.template in templates and not \
                 (len(set(templates[source.template]) ^ set(source.comps)) == 0\
                  or (len(set(source.comps) ^ self.distro.enabled_comps) == 0))):
                 self.distro.get_source_code = False
@@ -312,19 +319,19 @@ class SoftwareProperties(object):
                            ("Comps:", source.comps),
                            ("Enabled:", not source.disabled),
                            ("Valid:", not source.invalid)]:
-        print " %s %s" % (label, value)
+        print(" %s %s" % (label, value))
     if source.template:
         for (label, value) in [("MatchURI:", source.template.match_uri),
                                ("BaseURI:", source.template.base_uri)]:
-            print " %s %s" % (label, value)
-    print "\n"
+            print(" %s %s" % (label, value))
+    print("\n")
 
   def massive_debug_output(self):
     """Print the complete sources.list""" 
-    print "START SOURCES.LIST:"
+    print("START SOURCES.LIST:")
     for source in self.sourceslist:
-        print source.str()
-    print "END SOURCES.LIST\n"
+        print(source.str())
+    print("END SOURCES.LIST\n")
 
   def change_main_download_server(self, server):
     """ change the main download server """
@@ -519,7 +526,7 @@ class SoftwareProperties(object):
       has_template = 1
       has_comment = 1
       is_source = 1
-      revert_numbers = string.maketrans("0123456789", "9876543210")
+      revert_numbers = maketrans("0123456789", "9876543210")
       if source.template:
         has_template = 0
         desc = source.template.description
@@ -596,7 +603,7 @@ class SoftwareProperties(object):
       if os.path.isfile(f):
         break
     else:
-      print "No config found, creating one"
+      print("No config found, creating one")
       open(conffiles[0], "w")
 
     # ensure /etc/cron.daily/apt is executable
@@ -604,8 +611,8 @@ class SoftwareProperties(object):
     if os.path.exists(ac):
       perm = os.stat(ac)[stat.ST_MODE]
       if not (perm & stat.S_IXUSR):
-        print "file '%s' not executable, fixing" % ac
-        os.chmod(ac, 0755)
+        print("file '%s' not executable, fixing" % ac)
+        os.chmod(ac, 0o755)
 
     # now update them
     for periodic in conffiles:
@@ -684,7 +691,7 @@ class SoftwareProperties(object):
     repo = line.split()[0]
     try:
         areas = line.split(" ",1)[1]
-    except IndexError, e:
+    except IndexError:
         areas = "main"
     line = "deb %s %s %s" % ( repo, self.distro.codename, areas )
     return line
@@ -694,23 +701,14 @@ class SoftwareProperties(object):
     Add a source with the given apt line and auto-add
     signing key if we have it in the whitelist
     """
-    (deb_line, file) = expand_shortcut_line(line.strip(), self.distro.codename)
+    (deb_line, file) = expand_ppa_line(line.strip(), self.distro.codename)
     deb_line = self.expand_http_line(deb_line)
     debsrc_line = 'deb-src' + deb_line[3:]
     new_deb_entry = SourceEntry(deb_line, file)
     new_debsrc_entry = SourceEntry(debsrc_line, file)
     if new_deb_entry.invalid or new_debsrc_entry.invalid:
       return False
-
-    if line.startswith("cloud-archive:"):
-        # in trunk, we modified check_and_add_key_for_whitelisted_channels to
-        # take the line itself.  but this is that functions only caller
-        # so it should be safe and less invasive to do this check here instead.
-        ca_handler = ca_shortcut_handler(line.strip())
-        worker = threading.Thread(target=ca_handler.add_key)
-        worker.start()
-    else:
-        worker = self.check_and_add_key_for_whitelisted_channels(deb_line)
+    worker = self.check_and_add_key_for_whitelisted_channels(deb_line)
     self.sourceslist.add(new_deb_entry.type,
                          new_deb_entry.uri,
                          new_deb_entry.dist,
@@ -747,8 +745,8 @@ class SoftwareProperties(object):
 
   def add_key(self, path):
     """Add a gnupg key to the list of trusted software vendors"""
-    if not isinstance(path, unicode):
-      path = unicode(path) # allows non-ascii filenames
+    if not isinstance(path, str):
+      path = str(path) # allows non-ascii filenames
     if not os.path.exists(path):
         return False
     try:
@@ -761,7 +759,7 @@ class SoftwareProperties(object):
   def add_key_from_data(self, keydata):
     "Add a gnupg key from a data string (e.g. copy-n-paste)"
     tmp = tempfile.NamedTemporaryFile()
-    tmp.write(keydata)
+    tmp.write(keydata.encode())
     tmp.flush()
     return self.add_key(tmp.name)
 
@@ -784,15 +782,7 @@ class SoftwareProperties(object):
       return False
 
 
-def expand_shortcut_line(abrev, codename):
-    if abrev.startswith("cloud-archive:"):
-        handler = ca_shortcut_handler(abrev)
-        return handler.expand(codename)
-
-    return expand_ppa_line(abrev, distro_codename=codename)
-
-
 if __name__ == "__main__":
   sp = SoftwareProperties()
-  print sp.get_release_upgrades_policy()
+  print(sp.get_release_upgrades_policy())
   sp.set_release_upgrades_policy(0)
