@@ -1,5 +1,7 @@
 #!/usr/bin/python
 
+from __future__ import print_function
+
 import aptsources.distro
 import aptsources.sourceslist
 import unittest
@@ -11,8 +13,8 @@ class TestAptSources(unittest.TestCase):
 
     def setUp(self):
         apt_pkg.init()
-        apt_pkg.Config.Set("Dir::Etc", os.getcwd())
-        apt_pkg.Config.Set("Dir::Etc::sourceparts",".")
+        apt_pkg.config.set("Dir::Etc", os.getcwd())
+        apt_pkg.config.set("Dir::Etc::sourceparts",".")
 
     def testIsMirror(self):
         self.assertTrue(aptsources.sourceslist.is_mirror("http://archive.ubuntu.com",
@@ -21,7 +23,7 @@ class TestAptSources(unittest.TestCase):
                                               "http://ftp.debian.org"))
 
     def testSourcesListReading(self):
-        apt_pkg.Config.Set("Dir::Etc::sourcelist","data/sources.list")
+        apt_pkg.config.set("Dir::Etc::sourcelist","data/sources.list")
         sources = aptsources.sourceslist.SourcesList()
         self.assertEqual(len(sources.list), 6)
         # test load
@@ -30,7 +32,7 @@ class TestAptSources(unittest.TestCase):
         self.assertEqual(len(sources.list), 6)
 
     def testSourcesListAdding(self):
-        apt_pkg.Config.Set("Dir::Etc::sourcelist","data/sources.list")
+        apt_pkg.config.set("Dir::Etc::sourcelist","data/sources.list")
         sources = aptsources.sourceslist.SourcesList()
         # test to add something that is already there (main)
         before = copy.deepcopy(sources)
@@ -73,12 +75,12 @@ class TestAptSources(unittest.TestCase):
                         found_universe += 1
                     if c == "something":
                         found_something += 1
-        #print "\n".join([s.str() for s in sources])
+        #print("\n".join([s.str() for s in sources]))
         self.assertEqual(found_something, 1)
         self.assertEqual(found_universe, 1)
 
     def testDistribution(self):
-        apt_pkg.Config.Set("Dir::Etc::sourcelist","data/sources.list.testDistribution")
+        apt_pkg.config.set("Dir::Etc::sourcelist","data/sources.list.testDistribution")
         sources = aptsources.sourceslist.SourcesList()
         distro = aptsources.distro.get_distro(codename="edgy")
         distro.get_sources(sources)
@@ -87,7 +89,7 @@ class TestAptSources(unittest.TestCase):
         for s in sources:
             if s.template:
                 dist_templates.add(s.template.name)
-        #print dist_templates
+        #print(dist_templates)
         for d in ["edgy","edgy-security","edgy-updates","hoary"]:
             self.assertTrue(d in dist_templates)
         # test enable 
@@ -100,10 +102,10 @@ class TestAptSources(unittest.TestCase):
                 "edgy" in entry.dist):
                 for c in entry.comps:
                     if c == comp:
-                        if not found.has_key(entry.dist):
+                        if entry.dist not in found:
                             found[entry.dist] = 0
                         found[entry.dist] += 1
-        #print "".join([s.str() for s in sources])
+        #print("".join([s.str() for s in sources]))
         for key in found:
             self.assertEqual(found[key], 1)
 
@@ -117,10 +119,10 @@ class TestAptSources(unittest.TestCase):
                 entry.template.name == "edgy"):
                 for c in entry.comps:
                     if c == comp:
-                        if not found.has_key(entry.dist):
+                        if entry.dist not in found:
                             found[entry.dist] = 0
                         found[entry.dist] += 1
-        #print "".join([s.str() for s in sources])
+        #print("".join([s.str() for s in sources]))
         for key in found:
             self.assertEqual(found[key], 1)
 
